import React, { ReactElement, useRef, useState } from "react";
import { FlatList, StyleSheet, Text, TouchableOpacity, Modal, View } from "react-native";
import { Icon } from "@rneui/themed";
import { colorPalette } from "./Theme";

interface Props {
  label: string;
  data: Array<{ label: string; value: string }>;
  onSelect: (item: { label: string; value: string }) => void;
  error?: string;
}

export function Dropdown({ label, data, onSelect, error }: Props) {
  const DropdownButton = useRef<any>();
  const [visible, setVisible] = useState(false);
  const [selected, setSelected] = useState(undefined);
  const [dropdownTop, setDropdownTop] = useState(0);

  const toggleDropdown = (): void => {
    visible ? setVisible(false) : openDropdown();
  };

  const openDropdown = (): void => {
    DropdownButton.current.measure((_fx, _fy, _w, h, _px, py) => {
      setDropdownTop(py + h / 2);
    });
    setVisible(true);
  };

  const onItemPress = (item): void => {
    setSelected(item);
    onSelect(item);
    setVisible(false);
  };

  const renderItem = ({ item }): ReactElement<any, any> => (
    <TouchableOpacity
      style={styles.itemContainer}
      onPress={() => onItemPress(item)}
    >
      <Text style={styles.itemText}>{item.label}</Text>
    </TouchableOpacity>
  );

  const renderDropdown = (): ReactElement<any, any> => {
    return (
      <Modal visible={visible} transparent animationType="none">
        <TouchableOpacity
          style={styles.overlay}
          onPress={() => setVisible(false)}
        >
          <View style={[styles.dropdown, { top: dropdownTop }]}>
            <FlatList
              data={data}
              renderItem={renderItem}
              keyExtractor={(item, index) => index.toString()}
            />
          </View>
        </TouchableOpacity>
      </Modal>
    );
  };

  return (
    <>
      <TouchableOpacity
        ref={DropdownButton}
        style={styles.button}
        onPress={toggleDropdown}
      >
        {renderDropdown()}

        {selected && <Text style={styles.buttonText}>{selected.label}</Text>}
        {!selected && <Text style={styles.placeholderText}>{label}</Text>}

        {visible ? (
          <Icon
            type="font-awesome"
            name="chevron-up"
            color={colorPalette.neutral[600]}
          />
        ) : (
          <Icon
            type="font-awesome"
            name="chevron-down"
            color={colorPalette.neutral[600]}
          />
        )}
      </TouchableOpacity>
      {error && <Text style={styles.error}>{error}</Text>}
    </>
  );
}

const styles = StyleSheet.create({
  button: {
    backgroundColor: colorPalette.neutral[100],
    width: "85%",
    height: 64,
    paddingHorizontal: 24,
    borderRadius: 16,
    marginVertical: 5,
    display: "flex",
    flexDirection: "row",
    justifyContent: "space-between",
    alignItems: "center",
  },
  buttonText: {
    color: colorPalette.neutral[900],
    fontSize: 16,
    fontWeight: "400",
    letterSpacing: 0.75,
  },
  placeholderText: {
    color: colorPalette.neutral[600],
    fontSize: 16,
    fontWeight: "400",
    letterSpacing: 0.75,
  },
  dropdown: {
    marginHorizontal: 24,
    backgroundColor: "#fff",
    width: 310,
    shadowColor: "#000000",
    shadowRadius: 4,
    shadowOffset: { height: 4, width: 4 },
    shadowOpacity: 0.5,
    borderColor: "#e1e2e3",
    borderRadius: 2,
    borderRightWidth: 1,
    borderTopWidth: 1,
    borderLeftWidth: 1,
  },
  overlay: {
    width: "100%",
    height: "100%",
  },
  itemContainer: {
    paddingHorizontal: 12,
    paddingVertical: 10,
    borderBottomWidth: 1,
    borderColor: "#e1e2e3",
  },
  itemText: {
    fontSize: 16,
  },
  error: {
    color: "red",
    alignSelf: "flex-start",
    paddingHorizontal: 24,
    marginTop: -5,
  },
});

export default Dropdown;
