import React, { useEffect, useState } from 'react';
import { ScrollView, StyleSheet, Text, TouchableOpacity, View } from 'react-native';
import firebase from 'firebase/app';
import 'firebase/firestore';
import { useUser } from '../services/hooks/useUser';
import { api } from '../services/api';
import { AxiosError } from 'axios';
import { callToast } from '../utils/toast';
import { FontAwesome5 } from '@expo/vector-icons';
import { useNavigation } from '@react-navigation/native';
import { AppButton } from '../components/AppButton';

export function ChatList() {
    const [chat, setChat] = useState([]);
    const firestore = firebase.firestore();
    const { user } = useUser();
    const navigation = useNavigation();

    useEffect(() => {
        const getData = async () => {
            const chatsRef = firestore.collection('chats');
            const snapshot = await chatsRef.where("patient", "==", user.id).get();
            if (snapshot.empty) {
                console.log('No matching documents.');
                return;
            }
            api.get('/clinics/user-details').then((response) => {
                var specialists = response.data;
                var msgs = [];
                snapshot.forEach(async doc => {
                    firestore.collection('chats').doc(doc.id).collection('messages').orderBy('timestamp', 'desc').limit(1).onSnapshot((querySnapshot) => {
                        querySnapshot.forEach((msgDoc) => {
                            var specialistName = specialists.find(s => s.specialistId == doc.data().specialist)?.specialistName;
                            var oldMessage = msgs.find(m => m.name == specialistName)
                            if (specialistName) {
                                msgs = msgs.filter(m => m != oldMessage)
                                msgs = [...msgs, {
                                    timeStamp: new firebase.firestore.Timestamp(msgDoc.data().timestamp.seconds, msgDoc.data().timestamp.nanoseconds).toDate(),
                                    id: doc.data().specialist,
                                    name: specialistName,
                                    text: (msgDoc.data().sender == user.id ? 'Você: ' : '') + msgDoc.data().content,
                                }];
                                msgs.sort((a, b) => b.timeStamp - a.timeStamp)
                                setChat(msgs)
                            }
                        });
                    });
                });
            }).catch((error: AxiosError) => callToast(error.code as string));
        }
        getData();
    }, []);

    return (
        <>
            <ScrollView>
                {
                    chat.map((c, index) => {
                        return <TouchableOpacity
                            key={index}
                            style={styles.card}
                            onPress={() => navigation.navigate("Chat", {
                                id: c.id,
                                name: c.name
                            })}
                        >
                            <View style={{ backgroundColor: '#5D7684', padding: 15, borderRadius: 50 }}>
                                <FontAwesome5 name="user-alt" color={'white'} size={30} />
                            </View>
                            <View style={{ marginHorizontal: 12 }}>
                                <Text style={{ fontWeight: 'bold', fontSize: 16 }}>{c.name}</Text>
                                <Text style={{ fontSize: 15, marginRight: 18 }} numberOfLines={2}>{c.text}</Text>
                            </View>
                        </TouchableOpacity>
                    })
                }

            </ScrollView>
            <View style={{ width: "85%", alignSelf: "center", marginRight: "7%", paddingBottom: "2%" }}>
                <AppButton buttonLabel={'Ver mais especialistas'} onPress={() => navigation.navigate('TalkToSpecialist')} />
            </View>
        </>
    );
}

const styles = StyleSheet.create({
    card: {
        paddingHorizontal: 30,
        paddingVertical: 15,
        backgroundColor: 'rgba(109, 79, 211, 0.05)',
        marginBottom: 10,
        display: 'flex',
        flexDirection: 'row',
    },
});