import React, { useState } from "react";
import { useNavigation } from "@react-navigation/native";
import { InputDefault } from "../components/InputDefault";
import { User } from "../services/hooks/useUser";
import { CheckBox, ThemeProvider } from "@rneui/themed";
import { SafeAreaView, ScrollView, StyleSheet } from "react-native";
import { AppButton } from "../components/AppButton";
import { Dropdown } from "../components/Dropdown";
import { InputMasked } from "../components/InputMasked";
import { colorPalette, lightTheme } from "../components/Theme";
import { isInputNotEmpty, isPhoneValid } from "../utils/validations";
import { api } from "../services/api";
import { AxiosError } from "axios";
import { callToast } from "../utils/toast";

export type Clinic = {
  isHomeOffice?: boolean;
  name?: string;
  workPhone?: string;
  occupationArea?: string;
  cep?: string;
  state?: string;
  city?: string;
  district?: string;
  street?: string;
  addressNumber?: string;
  complement?: string;
};

const DUMMY_ESPECIALIZACOES = [
  { label: "Cardiologia", value: "1" },
  { label: "Neurologia", value: "2" },
  { label: "Pediatria", value: "3" },
  { label: "Pneumologia", value: "4" },
  { label: "Psiquiatria", value: "5" },
];

interface SpecialistStepFiveProps {
  setStep: React.Dispatch<React.SetStateAction<number>>;
  setUserData: React.Dispatch<React.SetStateAction<User>>;
  userData: User;
}

export function ClinicRegistration({ setStep, setUserData, userData }: SpecialistStepFiveProps) {
  const navigation = useNavigation();
  const [clinicData, setClinicData] = useState<Clinic>({});
  const [isHomeOffice, setIsHomeOffice] = useState(true);
  const [workPhoneError, setworkPhoneError] = useState("");
  const [occupationAreaError, setOccupationAreaError] = useState("");
  const [nameError, setNameError] = useState("");
  const [cepError, setCepError] = useState("");
  const [stateError, setStateError] = useState("");
  const [cityError, setCityError] = useState("");
  const [districtError, setDistrictError] = useState("");
  const [streetError, setStreetError] = useState("");
  const [addressNumberError, setAddressNumberError] = useState("");

  async function inputValidation() {
    const occupationAreaValidation = isInputNotEmpty(clinicData.occupationArea);
    occupationAreaValidation !== "valid" ? setOccupationAreaError(occupationAreaValidation) : setOccupationAreaError("");

    const workPhoneValidation = isPhoneValid(clinicData.workPhone);
    workPhoneValidation !== "valid" ? setworkPhoneError(workPhoneValidation) : setworkPhoneError("");

    if (!isHomeOffice) {
      const nameValidation = isInputNotEmpty(clinicData.name);
      nameValidation !== "valid" ? setNameError(nameValidation) : setNameError("");

      const cepValidation = isInputNotEmpty(clinicData.cep);
      cepValidation !== "valid" ? setCepError(cepValidation) : setCepError("");

      const stateValidation = isInputNotEmpty(clinicData.state);
      stateValidation !== "valid" ? setStateError(stateValidation) : setStateError("");

      const cityValidation = isInputNotEmpty(clinicData.city);
      cityValidation !== "valid" ? setCityError(cityValidation) : setCityError("");

      const districtValidation = isInputNotEmpty(clinicData.district);
      districtValidation !== "valid" ? setDistrictError(districtValidation) : setDistrictError("");

      const streetValidation = isInputNotEmpty(clinicData.street);
      streetValidation !== "valid" ? setStreetError(streetValidation) : setStreetError("");

      const addressNumberValidation = isInputNotEmpty(clinicData.addressNumber);
      addressNumberValidation !== "valid" ? setAddressNumberError(addressNumberValidation) : setAddressNumberError("");
      if (!isHomeOffice && workPhoneValidation === "valid" &&
        occupationAreaValidation === "valid" && nameValidation === "valid" &&
        workPhoneValidation === "valid" && cepValidation === "valid" &&
        stateValidation === "valid" && cityValidation === "valid" &&
        districtValidation === "valid" && streetValidation === "valid" &&
        addressNumberValidation === "valid") {
        createClinic();
      }
    }
    if (isHomeOffice && workPhoneValidation === "valid" &&
      occupationAreaValidation === "valid") {
      createClinic();
    }
  }

  async function createClinic() {
    const newClinic = { ...clinicData, isHomeOffice }
    await api.post('/clinics', newClinic).then((response) => {
      if (response.status === 200) {
        callToast("Clínica criada com sucesso!")
        navigation.navigate("TabScreens", { screen: "Home", initial: false, });
      }
    }).catch((error: AxiosError) => {
      callToast(error.response.data as string)
    });
  }
  return (
    <ThemeProvider theme={lightTheme}>
      <SafeAreaView style={styles.container}>
        <ScrollView
          style={styles.scrollView}
          contentContainerStyle={styles.contentContainer}
        >
          <CheckBox
            title="Home Office"
            checked={isHomeOffice}
            onPress={() => { setIsHomeOffice(!isHomeOffice); }}
          />
          <InputMasked
            placeholder="Celular de trabalho"
            value={clinicData.workPhone}
            onChangeText={(text) =>
              setClinicData({ ...clinicData, workPhone: text })
            }
            keyboardType="numeric"
            type={"cel-phone"}
            options={{
              maskType: "BRL",
              withDDD: true,
              dddMask: "(99) ",
            }}
            error={workPhoneError}
          />
          <Dropdown
            label="Área de atuação"
            data={DUMMY_ESPECIALIZACOES}
            onSelect={(item) =>
              setClinicData({ ...clinicData, occupationArea: item.label })
            }
            error={occupationAreaError}
          />
          {!isHomeOffice && (
            <>
              <InputDefault
                placeholder="Nome do local"
                value={clinicData.name}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, name: text })
                }
                error={nameError}
              />
              <InputMasked
                placeholder="CEP"
                value={clinicData.cep}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, cep: text })
                }
                keyboardType="numeric"
                type={"custom"}
                options={{
                  mask: "99999-999",
                }}
                error={cepError}
              />
              <InputDefault
                placeholder="Estado"
                value={clinicData.state}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, state: text })
                }
                error={stateError}
              />
              <InputDefault
                placeholder="Cidade"
                value={clinicData.city}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, city: text })
                }
                error={cityError}
              />
              <InputDefault
                placeholder="Bairro"
                value={clinicData.district}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, district: text })
                }
                error={districtError}
              />
              <InputDefault
                placeholder="Rua"
                value={clinicData.street}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, street: text })
                }
                error={streetError}
              />
              <InputDefault
                placeholder="Número"
                value={clinicData.addressNumber}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, addressNumber: text })
                }
                error={addressNumberError}
              />
              <InputDefault
                placeholder="Complemento"
                value={clinicData.complement}
                onChangeText={(text) =>
                  setClinicData({ ...clinicData, complement: text })
                }
              />
            </>
          )}
          <AppButton buttonLabel="Cadastrar" onPress={inputValidation} style={{ width: "85%" }} />
        </ScrollView>
      </SafeAreaView>
    </ThemeProvider>
  );
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
  },
  scrollView: {
    backgroundColor: colorPalette.neutral[0],
    height: "100%",
  },
  contentContainer: {
    display: "flex",
    flexDirection: "column",
    alignItems: "center",
  },
});
