import { useNavigation } from '@react-navigation/native';
import { ThemeProvider } from '@rneui/themed';
import React, { useEffect, useState } from 'react';
import { StyleSheet, SafeAreaView, ScrollView } from 'react-native';
import { User, useUser } from '../services/hooks/useUser';
import { isNameValid, isPhoneValid, isEmailValid } from '../utils/validations';
import { AppButton } from '../components/AppButton';
import { InputDefault } from '../components/InputDefault';
import { InputMasked } from '../components/InputMasked';
import { colorPalette, lightTheme } from '../components/Theme';
import { Logo } from '../components/Logo';
import { api } from '../services/api';
import { AxiosError } from 'axios';
import { callToast } from '../utils/toast';

export function EditUserInfo() {
  const [userData, setUserData] = useState<User>({});
  const [nameError, setNameError] = useState('');
  const [phoneError, setPhoneError] = useState('');
  const [responsibleNameError, setResponsibleNameError] = useState('');
  const [emailError, setEmailError] = useState('');
  const { user, setUser } = useUser();
  const navigation = useNavigation();

  useEffect(() => {
    setUserData(user)
  }, [])

  function inputValidation() {
    const nameValidation = isNameValid(userData.name)
    nameValidation !== 'valid' ?
      setNameError(nameValidation) : setNameError('')

    const responsibleNameValidation = isNameValid(userData.responsibleName)
    responsibleNameValidation !== 'valid' ?
      setResponsibleNameError(responsibleNameValidation) : setResponsibleNameError('')

    const phoneValidation = isPhoneValid(userData.phone)
    phoneValidation !== 'valid' ?
      setPhoneError(phoneValidation) : setPhoneError('')

    const emailValidation = isEmailValid(userData.email)
    emailValidation !== 'valid' ?
      setEmailError(emailValidation) : setEmailError('')

    if (nameValidation === 'valid' && phoneValidation === 'valid' &&
      emailValidation === 'valid') {
      api.put("/users", userData)
        .then(response => {
          setUser(response.data);
          callToast("Informações alteradas com sucesso.");
          navigation.goBack();
        })
        .catch((error: AxiosError) => callToast(error.response.data as string))
    }
  }

  return (
    <ThemeProvider theme={lightTheme}>
      <SafeAreaView style={styles.container}>
        <ScrollView
          style={styles.scrollView}
          contentContainerStyle={styles.contentContainer}
        >
          <Logo margin={true} />
          <InputDefault
            placeholder='Nome completo'
            value={userData.name}
            onChangeText={text => setUserData({ ...userData, name: text })}
            error={nameError}
          />
          <InputMasked placeholder='Celular'
            keyboardType="numeric"
            type={"cel-phone"}
            value={userData.phone}
            options={{
              maskType: 'BRL',
              withDDD: true,
              dddMask: '(99) '
            }}
            onChangeText={text => setUserData({ ...userData, phone: text })}
            error={phoneError}
          />
          <InputDefault
            placeholder='E-mail'
            keyboardType="email-address"
            value={userData.email}
            onChangeText={text => setUserData({ ...userData, email: text })}
            error={emailError}
          />
          {user.userType === "PATIENT" ?
            <InputDefault
              placeholder='Nome completo do responsável'
              value={userData.responsibleName}
              onChangeText={text => setUserData({ ...userData, responsibleName: text })}
              error={responsibleNameError}
            />
            : <></>
          }
          <AppButton
            buttonLabel="Alterar informações"
            onPress={inputValidation}
            style={{ width: "85%" }}
          />
        </ScrollView>
      </SafeAreaView>
    </ThemeProvider>
  )
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: colorPalette.neutral[0],
  },
  scrollView: {
    backgroundColor: colorPalette.neutral[0],
    height: "100%",
  },
  contentContainer: {
    display: "flex",
    flexDirection: "column",
    alignItems: "center",
  },
});