import React, { useEffect, useState } from "react";
import {
  Text,
  View,
  StyleSheet,
  KeyboardAvoidingView,
  Platform,
  Image,
} from "react-native";
import { isDiaryValid } from "../utils/validations";
import { AppButton } from "../components/AppButton";
import { InputDefault } from "../components/InputDefault";
import { useNavigation, useRoute } from "@react-navigation/native";
import { colorPalette } from "../components/Theme";
import { DiaryDayParams } from "../@types/navigation";
import { api } from "../services/api";
import { AxiosError } from "axios";
import { callToast } from "../utils/toast";

export type Diary = {
  date?: String;
  text?: string;
};

export function PatientDiaryDay() {
  const navigation = useNavigation();
  const [diaryData, setDiaryData] = useState<Diary>({});
  const router = useRoute();
  const { date } = router.params as DiaryDayParams;
  const [textError, setTextError] = useState("");

  useEffect(() => {
    api.get(`/diaries/date?date=${date}`).then((response) => {
      setDiaryData(response.data)
    }).catch()
  }, [date])

  function inputValidation() {
    const diaryValidation = isDiaryValid(diaryData.text);
    diaryValidation !== "valid"
      ? setTextError(diaryValidation) : setTextError("");

    if (diaryValidation === "valid") {
      saveDiary();
    }
  }

  async function saveDiary() {
    const newDiary = { ...diaryData, date }
    await api.post('/diaries', newDiary).then((response) => {
      if (response.status === 200) {
        callToast("Diário criado com sucesso!")
        navigation.goBack();
      }
    }).catch((error: AxiosError) => {
      callToast(error.response.data as string)
    });
  }

  return (
    <KeyboardAvoidingView
      behavior={Platform.OS === "ios" ? "padding" : "height"}
      keyboardVerticalOffset={120}
      style={styles.container}
    >
      <View style={styles.calendar}>
        <Image
          source={require("../assets/images/calendar_icon.png")}
          style={styles.imageStyle}
        />
        <Text style={styles.textStyle}>{date}</Text>
      </View>
      <InputDefault
        style={styles.input}
        placeholder="Escreva sobre o dia da criança. O que ela fez, como se sentiu, etc..."
        value={diaryData.text}
        onChangeText={(text) =>
          setDiaryData({ ...diaryData, text: text })
        }
        scrollEnabled={false}
        multiline={true}
        maxLength={500}
        error={textError}
      />
      <AppButton
        buttonLabel="Salvar"
        onPress={inputValidation}
        style={{ width: "85%" }}
      />
    </KeyboardAvoidingView>
  );
}

const styles = StyleSheet.create({
  container: {
    height: "100%",
    width: "100%",
    display: "flex",
    alignItems: "center",
    justifyContent: "flex-start",
    backgroundColor: "white",
    flex: 1,
  },
  input: {
    backgroundColor: "white",
    margin: 10,
    width: "83%",
    height: "65%",
    fontSize: 16,
    textAlignVertical: "top",
  },
  textStyle: {
    textAlign: "center",
    fontSize: 18,
    color: 'black',
  },
  calendar: {
    flexDirection: "row",
    backgroundColor: colorPalette.neutral[100],
    justifyContent: "flex-start",
    marginTop: 25,
    marginBottom: 10,
    borderRadius: 7,
    width: "85%",
    height: 64,
    fontSize: 20,
    alignItems: "center",
  },
  imageStyle: {
    padding: 10,
    margin: 20,
    height: 20,
    width: 20,
    resizeMode: "stretch",
  },
});
