import { AxiosError } from 'axios';
import React, { createContext, ReactNode, useContext, useState } from 'react';
import { callToast } from '../../utils/toast';
import { api } from '../api';
import * as SecureStore from 'expo-secure-store';

export interface User {
  id?: number;
  name?: string;
  email?: string;
  password?: string;
  responsibleName?: string;
  phone?: string;
  birthday?: string;
  cpf?: string;
  crm?: string;
  userType?: "PATIENT" | "SPECIALIST";
}

interface LoginInput {
  email: string;
  password: string;
}

interface UserProviderProp {
  children: ReactNode;
}

interface UserContextData {
  user?: User;
  createUser: (transaction: User) => Promise<number>;
  setUser: (newState: Partial<User>) => void;
  login: (input: LoginInput) => Promise<number>;
}

const UserContext = createContext<UserContextData>(
  {} as UserContextData
);

export function UserProvider({ children }: UserProviderProp) {
  const [user, setUser] = useState<User>({});

  async function createUser(userInput: User): Promise<number> {
    let responseCode;
    await api.post('/users', userInput).then((response) => {
      setUser(response.data)
      responseCode = response.status
    }).catch((error: AxiosError) => {
      callToast(error.response.data as string)
      responseCode = error.code
    });
    return responseCode;
  }

  async function login(loginInput: LoginInput) {
    let responseCode;
    await api.post('/login', loginInput).then((response) => {
      SecureStore.setItemAsync('access_token', response.data.authorization)
      setUser(response.data)
      responseCode = response.status
    }).catch((error: AxiosError) => {
      console.log(error)
      if (error.response.status === 401)
        callToast("Credenciais inválidas")
      responseCode = error.code
    });
    return responseCode;
  }

  return (
    <UserContext.Provider value={{ user, createUser, setUser, login }} >
      {children}
    </UserContext.Provider>
  );
}

export function useUser() {
  const context = useContext(UserContext);

  return context;
}